// Wavey grid monster!! - DELTRON 5/12/04
// - demonstrates a simple way to draw a surface

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(linker, "/SUBSYSTEM:WINDOWS /ENTRY:mainCRTStartup")

#include <windows.h>
#include <stdio.h>
#include <GL\glut.h>
#include <GL\gl.h>
#include <mmsystem.h>							// multi media timer support
#include <time.h>								// for help manipulating time
#include <math.h>

// GLOBALS
const int WINDOW_X = 700;	// size of window
const int WINDOW_Y = 700;	//
clock_t	initialclock;


// a cheap way to show off the verts
void nextcolor()
{
	float r,g,b;
	r=0.2f + (float)(rand()%70)/100;
	g=0.2f + (float)(rand()%70)/100;
	b=0.2f + (float)(rand()%70)/100;
	glColor4f(r,g,b,1);
}


// The data is in this form because we need unique verts to modify.
// If we did not have exposure to unique verts, we'd have to keep
// track of shared verts so that the surface wouldn't form cracks.

GLfloat unique_verts[][3] = {
        {1,-1,0}, {0,-1,0}, {0,0,0}, {1,0,0}, {-1,-1,0},
        {-1,0,0}, {0,1,0}, {1,1,0}, {-1,1,0}
};

GLuint indices[][4] = {
        {0,1,2,3}, {1,4,5,2}, {3,2,6,7}, {2,5,8,6}
};

void rendersurface()
{
	glBegin(GL_QUADS);
	for (int i = 0; i < 4; i++)
	{
		nextcolor(); glVertex3fv(&unique_verts[indices[i][0]][0]);
		nextcolor(); glVertex3fv(&unique_verts[indices[i][1]][0]);
		nextcolor(); glVertex3fv(&unique_verts[indices[i][2]][0]);
		nextcolor(); glVertex3fv(&unique_verts[indices[i][3]][0]);
        }
	glEnd();
}

// play with the z's
float ang=0;
void movesurface(void)
{
	for (int i=0; i<9; i++)
	{
		unique_verts[i][2]+=(sin(ang*i)/500);
		ang+=0.001f;
	}

}


// *** MAIN RENDER ***
void render_scene(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0,0,-6.0f);
	glRotatef(-70,1,1,1);
	srand(100);
	rendersurface();
}


void display_function(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
// render the scene
  render_scene();

// update the surface animation after so much time has passed
  clock_t currentclock = timeGetTime();
  if (currentclock-initialclock>5)
  {
	initialclock=currentclock;
	movesurface();
  }
// Check for GL errors 
  if(GLenum error = glGetError()) {MessageBox(NULL, (const char *)gluErrorString(error), "ERROR", MB_OK); exit(1);}

  glutSwapBuffers();
}


void idle(void)
{
  glutPostRedisplay();
}


void visible(int vis)
{
  if (vis == GLUT_VISIBLE)
    glutIdleFunc(idle);
  else
    glutIdleFunc(NULL);
}


void reshape(int w, int h)
{
	glViewport(0,0,w,h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, float(w)/float(h), 1, 30);
	glMatrixMode(GL_MODELVIEW);
}


static void key(unsigned char c, int x, int y)
{
  if (c == 27) {
    exit(0);
  }
  glutPostRedisplay();
}

int main(int argc, char **argv)
{
	GLsizei sx = glutGet( GLUT_SCREEN_WIDTH ); 
    GLsizei sy = glutGet( GLUT_SCREEN_HEIGHT );

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowPosition(sx/2-WINDOW_X/2, sy/2-WINDOW_Y/2);
	glutInitWindowSize(WINDOW_X, WINDOW_Y);
	glutCreateWindow("Wavey grid monster!! - DELTRON 5/12/04");
	glutKeyboardFunc(key);
	glutDisplayFunc(display_function);
	glutVisibilityFunc(visible);
	glutReshapeFunc(reshape);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, (WINDOW_X/WINDOW_Y), 1, 30);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	initialclock = timeGetTime();

	glutMainLoop();
	return 0;
}


